@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright 2018-2020,2021 Thomas E. Dickey                                *
+ * Copyright 2018-2024,2025 Thomas E. Dickey                                *
  * Copyright 1998-2016,2017 Free Software Foundation, Inc.                  *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
@@ -41,8 +41,11 @@
 **
 */
 
+#define NEW_PAIR_INTERNAL 1
+
 #include <curses.priv.h>
 #include <tic.h>
+#include <new_pair.h>
 
 #if USE_GPM_SUPPORT
 #ifdef HAVE_LIBDL
@@ -54,7 +57,7 @@
 #undef CUR
 #define CUR SP_TERMTYPE
 
-MODULE_ID("$Id: lib_set_term.c,v 1.179 2021/05/08 21:48:34 tom Exp $")
+MODULE_ID("$Id: lib_set_term.c,v 1.196 2025/02/15 12:55:53 tom Exp $")
 
 #ifdef USE_TERM_DRIVER
 #define MaxColors      InfoOf(sp).maxcolors
@@ -78,7 +81,7 @@ set_term(SCREEN *screenp)
     _nc_set_screen(screenp);
     newSP = screenp;
 
-    if (newSP != 0) {
+    if (newSP != NULL) {
 	TINFO_SET_CURTERM(newSP, newSP->_term);
 #if !USE_REENTRANT
 	curscr = CurScreen(newSP);
@@ -88,11 +91,11 @@ set_term(SCREEN *screenp)
 	COLOR_PAIRS = newSP->_pair_count;
 #endif
     } else {
-	TINFO_SET_CURTERM(oldSP, 0);
+	TINFO_SET_CURTERM(oldSP, NULL);
 #if !USE_REENTRANT
-	curscr = 0;
-	newscr = 0;
-	stdscr = 0;
+	curscr = NULL;
+	newscr = NULL;
+	stdscr = NULL;
 	COLORS = 0;
 	COLOR_PAIRS = 0;
 #endif
@@ -107,7 +110,7 @@ set_term(SCREEN *screenp)
 static void
 _nc_free_keytry(TRIES * kt)
 {
-    if (kt != 0) {
+    if (kt != NULL) {
 	_nc_free_keytry(kt->child);
 	_nc_free_keytry(kt->sibling);
 	free(kt);
@@ -117,7 +120,7 @@ _nc_free_keytry(TRIES * kt)
 static bool
 delink_screen(SCREEN *sp)
 {
-    SCREEN *last = 0;
+    SCREEN *last = NULL;
     SCREEN *temp;
     bool result = FALSE;
 
@@ -146,6 +149,9 @@ delscreen(SCREEN *sp)
 
     _nc_lock_global(curses);
     if (delink_screen(sp)) {
+	WINDOWLIST *wl;
+	bool is_current = (sp == CURRENT_SCREEN);
+
 #ifdef USE_SP_RIPOFF
 	if (safe_ripoff_sp && safe_ripoff_sp != safe_ripoff_stack) {
 	    ripoff_t *rop;
@@ -154,19 +160,23 @@ delscreen(SCREEN *sp)
 		 rop++) {
 		if (rop->win) {
 		    (void) delwin(rop->win);
-		    rop->win = 0;
+		    rop->win = NULL;
 		}
 	    }
 	}
 #endif
 
-	(void) _nc_freewin(CurScreen(sp));
-	(void) _nc_freewin(NewScreen(sp));
-	(void) _nc_freewin(StdScreen(sp));
+	/* delete all of the windows in this screen */
+      rescan:
+	for (each_window(sp, wl)) {
+	    if (_nc_freewin(&(wl->win)) == OK) {
+		goto rescan;
+	    }
+	}
 
-	if (sp->_slk != 0) {
+	if (sp->_slk != NULL) {
 
-	    if (sp->_slk->ent != 0) {
+	    if (sp->_slk->ent != NULL) {
 		int i;
 
 		for (i = 0; i < sp->_slk->labcnt; ++i) {
@@ -176,17 +186,18 @@ delscreen(SCREEN *sp)
 		free(sp->_slk->ent);
 	    }
 	    free(sp->_slk);
-	    sp->_slk = 0;
+	    sp->_slk = NULL;
 	}
 
 	_nc_free_keytry(sp->_keytry);
-	sp->_keytry = 0;
+	sp->_keytry = NULL;
 
 	_nc_free_keytry(sp->_key_ok);
-	sp->_key_ok = 0;
+	sp->_key_ok = NULL;
 
 	FreeIfNeeded(sp->_current_attr);
 
+	_nc_free_ordered_pairs(sp);
 	FreeIfNeeded(sp->_color_table);
 	FreeIfNeeded(sp->_color_pairs);
 
@@ -206,9 +217,9 @@ delscreen(SCREEN *sp)
 	}
 #if USE_GPM_SUPPORT
 #ifdef HAVE_LIBDL
-	if (sp->_dlopen_gpm != 0) {
+	if (sp->_dlopen_gpm != NULL) {
 	    dlclose(sp->_dlopen_gpm);
-	    sp->_dlopen_gpm = 0;
+	    sp->_dlopen_gpm = NULL;
 	}
 #endif
 #endif /* USE_GPM_SUPPORT */
@@ -219,21 +230,23 @@ delscreen(SCREEN *sp)
 	 * application might try to use (except cur_term, which may have
 	 * multiple references in different screens).
 	 */
-	if (sp == CURRENT_SCREEN) {
+	if (is_current) {
 #if !USE_REENTRANT
-	    curscr = 0;
-	    newscr = 0;
-	    stdscr = 0;
+	    curscr = NULL;
+	    newscr = NULL;
+	    stdscr = NULL;
 	    COLORS = 0;
 	    COLOR_PAIRS = 0;
 #endif
-	    _nc_set_screen(0);
+	    _nc_set_screen(NULL);
 #if USE_WIDEC_SUPPORT
-	    if (SP == 0) {
+	    if (SP == NULL) {
 		FreeIfNeeded(_nc_wacs);
-		_nc_wacs = 0;
+		_nc_wacs = NULL;
 	    }
 #endif
+	} else {
+	    set_term(CURRENT_SCREEN);
 	}
     }
     _nc_unlock_global(curses);
@@ -290,7 +303,7 @@ extract_fgbg(const char *src, int *result)
 }
 #endif
 
-#define ReturnScreenError() do { _nc_set_screen(0); \
+#define ReturnScreenError() do { _nc_set_screen(NULL); \
                             returnCode(ERR); } while (0)
 
 /* OS-independent screen initializations */
@@ -302,7 +315,7 @@ NCURSES_SP_NAME(_nc_setupscreen) (
 				     int slines,
 				     int scolumns,
 				     FILE *output,
-				     int filtered,
+				     bool filtered,
 				     int slk_format)
 {
 #ifndef USE_TERM_DRIVER
@@ -318,10 +331,13 @@ NCURSES_SP_NAME(_nc_setupscreen) (
     T((T_CALLED("_nc_setupscreen(%d, %d, %p, %d, %d)"),
        slines, scolumns, (void *) output, filtered, slk_format));
 
-    assert(CURRENT_SCREEN == 0);	/* has been reset in newterm() ! */
+    /* CURRENT_SCREEN is reset in newterm() */
+    if (CURRENT_SCREEN)
+	returnCode(ERR);
 
 #if NCURSES_SP_FUNCS
-    assert(spp != 0);
+    if (spp == NULL)
+	returnCode(ERR);
     sp = *spp;
 
     if (!sp) {
@@ -344,13 +360,13 @@ NCURSES_SP_NAME(_nc_setupscreen) (
     sp->_next_screen = _nc_screen_chain;
     _nc_screen_chain = sp;
 
-    if ((sp->_current_attr = typeCalloc(NCURSES_CH_T, 1)) == 0) {
+    if ((sp->_current_attr = typeCalloc(NCURSES_CH_T, 1)) == NULL) {
 	ReturnScreenError();
     }
 #else
     if (!_nc_alloc_screen()
-	|| ((SP->_acs_map = typeCalloc(chtype, ACS_LEN)) == 0)
-	|| ((SP->_screen_acs_map = typeCalloc(bool, ACS_LEN)) == 0)) {
+	|| ((SP->_acs_map = typeCalloc(chtype, ACS_LEN)) == NULL)
+	|| ((SP->_screen_acs_map = typeCalloc(bool, ACS_LEN)) == NULL)) {
 	returnCode(ERR);
     }
 
@@ -360,7 +376,7 @@ NCURSES_SP_NAME(_nc_setupscreen) (
     sp->_next_screen = _nc_screen_chain;
     _nc_screen_chain = sp;
 
-    if ((sp->_current_attr = typeCalloc(NCURSES_CH_T, 1)) == 0) {
+    if ((sp->_current_attr = typeCalloc(NCURSES_CH_T, 1)) == NULL) {
 	returnCode(ERR);
     }
 #endif
@@ -376,6 +392,10 @@ NCURSES_SP_NAME(_nc_setupscreen) (
 #else
     _nc_get_screensize(sp, &slines, &scolumns);
 #endif
+    if (scolumns < 0)
+	scolumns = 0;
+    if (slines < 0)
+	slines = 0;
     SET_LINES(slines);
     SET_COLS(scolumns);
 
@@ -407,11 +427,6 @@ NCURSES_SP_NAME(_nc_setupscreen) (
 #endif
 	T(("filter screensize %dx%d", slines, scolumns));
     }
-#ifdef __DJGPP__
-    T(("setting output mode to binary"));
-    fflush(output);
-    setmode(output, O_BINARY);
-#endif
 #if defined(EXP_WIN32_DRIVER)
     T(("setting output mode to binary"));
     fflush(output);
@@ -429,7 +444,7 @@ NCURSES_SP_NAME(_nc_setupscreen) (
 	_setmode(fileno(output), _O_BINARY);
 #endif
     sp->out_limit = (size_t) ((2 + slines) * (6 + scolumns));
-    if ((sp->out_buffer = malloc(sp->out_limit)) == 0)
+    if ((sp->out_buffer = malloc(sp->out_limit)) == NULL)
 	sp->out_limit = 0;
     sp->out_inuse = 0;
 
@@ -471,7 +486,7 @@ NCURSES_SP_NAME(_nc_setupscreen) (
      * Allow those assumed/default color assumptions to be overridden at
      * runtime:
      */
-    if ((env = getenv("NCURSES_ASSUMED_COLORS")) != 0) {
+    if ((env = getenv("NCURSES_ASSUMED_COLORS")) != NULL) {
 	int fg, bg;
 	char sep1, sep2;
 	int count = sscanf(env, "%d%c%d%c", &fg, &sep1, &bg, &sep2);
@@ -547,7 +562,7 @@ NCURSES_SP_NAME(_nc_setupscreen) (
      * in the environment, reset the support-flag.
      */
     if (magic_cookie_glitch >= 0) {
-	if (getenv("NCURSES_NO_MAGIC_COOKIE") != 0) {
+	if (getenv("NCURSES_NO_MAGIC_COOKIE") != NULL) {
 	    support_cookies = FALSE;
 	}
     }
@@ -625,10 +640,10 @@ NCURSES_SP_NAME(_nc_setupscreen) (
     NCURSES_SP_NAME(_nc_init_acs) (NCURSES_SP_ARG);
 #if USE_WIDEC_SUPPORT
     sp->_screen_unicode = _nc_unicode_locale();
-    if (_nc_wacs == 0) {
+    if (_nc_wacs == NULL) {
 	_nc_init_wacs();
     }
-    if (_nc_wacs == 0) {
+    if (_nc_wacs == NULL) {
 	ReturnScreenError();
     }
 
@@ -636,7 +651,7 @@ NCURSES_SP_NAME(_nc_setupscreen) (
 			   && _nc_locale_breaks_acs(sp->_term));
 #endif
     env = _nc_get_locale();
-    sp->_legacy_coding = ((env == 0)
+    sp->_legacy_coding = ((env == NULL)
 			  || !strcmp(env, "C")
 			  || !strcmp(env, "POSIX"));
     T(("legacy-coding %d", sp->_legacy_coding));
@@ -644,19 +659,19 @@ NCURSES_SP_NAME(_nc_setupscreen) (
     sp->_nc_sp_idcok = TRUE;
     sp->_nc_sp_idlok = FALSE;
 
-    sp->oldhash = 0;
-    sp->newhash = 0;
+    sp->oldhash = NULL;
+    sp->newhash = NULL;
 
     T(("creating newscr"));
     NewScreen(sp) = NCURSES_SP_NAME(newwin) (NCURSES_SP_ARGx slines, scolumns,
 					     0, 0);
-    if (NewScreen(sp) == 0) {
+    if (NewScreen(sp) == NULL) {
 	ReturnScreenError();
     }
     T(("creating curscr"));
     CurScreen(sp) = NCURSES_SP_NAME(newwin) (NCURSES_SP_ARGx slines, scolumns,
 					     0, 0);
-    if (CurScreen(sp) == 0) {
+    if (CurScreen(sp) == NULL) {
 	ReturnScreenError();
     }
 #if !USE_REENTRANT
@@ -676,7 +691,7 @@ NCURSES_SP_NAME(_nc_setupscreen) (
      * unless we use the term-driver.
      */
 #ifndef USE_TERM_DRIVER
-    if (cur_term != 0 &&
+    if (cur_term != NULL &&
 	!memcmp(&cur_term->Ottyb, &null_TTY, sizeof(TTY)))
 #endif
     {
@@ -738,10 +753,11 @@ NCURSES_SP_NAME(_nc_setupscreen) (
     }
 
     T(("creating stdscr"));
+    (void) bottom_stolen;
     assert((sp->_lines_avail + sp->_topstolen + bottom_stolen) == slines);
     if ((StdScreen(sp) = NCURSES_SP_NAME(newwin) (NCURSES_SP_ARGx
 						  sp->_lines_avail,
-						  scolumns, 0, 0)) == 0) {
+						  scolumns, 0, 0)) == NULL) {
 	ReturnScreenError();
     }
     SET_LINES(sp->_lines_avail);
@@ -757,10 +773,10 @@ NCURSES_EXPORT(int)
 _nc_setupscreen(int slines GCC_UNUSED,
 		int scolumns GCC_UNUSED,
 		FILE *output,
-		int filtered,
+		bool filtered,
 		int slk_format)
 {
-    SCREEN *sp = 0;
+    SCREEN *sp = NULL;
     int rc = NCURSES_SP_NAME(_nc_setupscreen) (&sp,
 					       slines,
 					       scolumns,
@@ -768,7 +784,7 @@ _nc_setupscreen(int slines GCC_UNUSED,
 					       filtered,
 					       slk_format);
     if (rc != OK)
-	_nc_set_screen(0);
+	_nc_set_screen(NULL);
     return rc;
 }
 #endif
@@ -791,13 +807,13 @@ NCURSES_SP_NAME(_nc_ripoffline) (NCURSES_SP_DCLx
        TR_FUNC_ARG(0, init)));
 
 #if NCURSES_SP_FUNCS
-    if (SP_PARM != 0 && SP_PARM->_prescreen)
+    if (SP_PARM != NULL && SP_PARM->_prescreen)
 #endif
     {
 	if (line == 0) {
 	    code = OK;
 	} else {
-	    if (safe_ripoff_sp == 0) {
+	    if (safe_ripoff_sp == NULL) {
 		safe_ripoff_sp = safe_ripoff_stack;
 	    }
 	    if (safe_ripoff_sp < safe_ripoff_stack + N_RIPS) {
@@ -819,10 +835,13 @@ NCURSES_EXPORT(int)
 _nc_ripoffline(int line, int (*init) (WINDOW *, int))
 {
     int rc;
+
+    _nc_init_pthreads();
     _nc_lock_global(prescreen);
     START_TRACE();
     rc = NCURSES_SP_NAME(_nc_ripoffline) (CURRENT_SCREEN_PRE, line, init);
     _nc_unlock_global(prescreen);
+
     return rc;
 }
 #endif
@@ -843,10 +862,13 @@ NCURSES_EXPORT(int)
 ripoffline(int line, int (*init) (WINDOW *, int))
 {
     int rc;
+
+    _nc_init_pthreads();
     _nc_lock_global(prescreen);
     START_TRACE();
     rc = NCURSES_SP_NAME(ripoffline) (CURRENT_SCREEN_PRE, line, init);
     _nc_unlock_global(prescreen);
+
     return rc;
 }
 #endif
