@@ -1,5 +1,5 @@
 /****************************************************************************
- * Copyright 2018-2020,2021 Thomas E. Dickey                                *
+ * Copyright 2018-2024,2025 Thomas E. Dickey                                *
  * Copyright 1998-2016,2017 Free Software Foundation, Inc.                  *
  *                                                                          *
  * Permission is hereby granted, free of charge, to any person obtaining a  *
@@ -40,7 +40,7 @@
 #include <termsort.h>		/* this C file is generated */
 #include <parametrized.h>	/* so is this */
 
-MODULE_ID("$Id: dump_entry.c,v 1.189 2021/09/04 10:29:59 tom Exp $")
+MODULE_ID("$Id: dump_entry.c,v 1.199 2025/02/08 21:53:40 tom Exp $")
 
 #define DISCARD(string) string = ABSENT_STRING
 #define PRINTF (void) printf
@@ -121,7 +121,7 @@ strncpy_DYN(DYNBUF * dst, const char *src, size_t need)
     if (want > dst->size) {
 	dst->size += (want + 1024);	/* be generous */
 	dst->text = typeRealloc(char, dst->size, dst->text);
-	if (dst->text == 0)
+	if (dst->text == NULL)
 	    failed("strncpy_DYN");
     }
     _nc_STRNCPY(dst->text + dst->used, src, need + 1);
@@ -132,7 +132,7 @@ strncpy_DYN(DYNBUF * dst, const char *src, size_t need)
 static void
 strcpy_DYN(DYNBUF * dst, const char *src)
 {
-    if (src == 0) {
+    if (src == NULL) {
 	dst->used = 0;
 	strcpy_DYN(dst, "");
     } else {
@@ -144,9 +144,9 @@ strcpy_DYN(DYNBUF * dst, const char *src)
 static void
 free_DYN(DYNBUF * p)
 {
-    if (p->text != 0)
+    if (p->text != NULL)
 	free(p->text);
-    p->text = 0;
+    p->text = NULL;
     p->size = 0;
     p->used = 0;
 }
@@ -170,7 +170,7 @@ nametrans(const char *name)
 {
     const struct name_table_entry *np;
 
-    if ((np = _nc_find_entry(name, _nc_get_hash_table(0))) != 0) {
+    if ((np = _nc_find_entry(name, _nc_get_hash_table(0))) != NULL) {
 	switch (np->nte_type) {
 	case BOOLEAN:
 	    NameTrans(bool_from_termcap, boolcodes);
@@ -186,7 +186,7 @@ nametrans(const char *name)
 	}
     }
 
-    return (0);
+    return (NULL);
 }
 
 void
@@ -212,7 +212,7 @@ dump_init(const char *version,
     did_wrap = (width <= 0);
 
     /* versions */
-    if (version == 0)
+    if (version == NULL)
 	tversion = V_ALLCAPS;
     else if (!strcmp(version, "SVr1") || !strcmp(version, "SVR1")
 	     || !strcmp(version, "Ultrix"))
@@ -458,7 +458,7 @@ op_length(const char *src, int offset)
 		}
 		n++;
 	    }
-	} else if (strchr("pPg", ch) != 0) {
+	} else if (strchr("pPg", ch) != NULL) {
 	    result += 2;
 	} else {
 	    result++;		/* ordinary operator */
@@ -526,7 +526,7 @@ fill_spaces(const char *src)
     const char *fill = "\\s";
     size_t need = strlen(src);
     size_t size = strlen(fill);
-    char *result = 0;
+    char *result = NULL;
     int pass;
     size_t s, d;
     for (pass = 0; pass < 2; ++pass) {
@@ -550,7 +550,7 @@ fill_spaces(const char *src)
 	    result[d] = '\0';
 	} else {
 	    result = calloc(need + 1, sizeof(char));
-	    if (result == 0)
+	    if (result == NULL)
 		failed("fill_spaces");
 	}
     }
@@ -598,7 +598,7 @@ wrap_concat(const char *src, int need, unsigned mode)
 	if (TcOutput())
 	    trailer = "\\\n\t ";
 
-	if (!TcOutput() && (p = strchr(fill, '=')) != 0) {
+	if (!TcOutput() && (p = strchr(fill, '=')) != NULL) {
 	    base = (int) (p + 1 - fill);
 	    if (base > 8)
 		base = 8;
@@ -612,7 +612,7 @@ wrap_concat(const char *src, int need, unsigned mode)
 	    align[base] = '\0';
 	}
 	/* "pretty" overrides wrapping if it already split the line */
-	if (!pretty || strchr(fill, '\n') == 0) {
+	if (!pretty || strchr(fill, '\n') == NULL) {
 	    int tag = 0;
 
 	    if (TcOutput() && outbuf.used && !wrap_1ST(mode)) {
@@ -725,7 +725,7 @@ indent_DYN(DYNBUF * buffer, int level)
  * given leading text.
  */
 static bool
-leading_DYN(DYNBUF * buffer, const char *leading)
+leading_DYN(const DYNBUF * buffer, const char *leading)
 {
     bool result = FALSE;
     size_t need = strlen(leading);
@@ -841,7 +841,7 @@ fmt_complex(TERMTYPE2 *tterm, const char *capability, char *src, int level)
 		    strncpy_DYN(&tmpbuf, src++, (size_t) 1);
 		    if (src[0] == '%'
 			&& src[1] != '\0'
-			&& (strchr("?e;", src[1])) == 0) {
+			&& (strchr("?e;", src[1])) == NULL) {
 			tmpbuf.text[tmpbuf.used++] = '\n';
 			indent_DYN(&tmpbuf, level);
 		    }
@@ -859,7 +859,6 @@ fmt_complex(TERMTYPE2 *tterm, const char *capability, char *src, int level)
 		indent_DYN(&tmpbuf, level + 1);
 		strncpy_DYN(&tmpbuf, "%", (size_t) 1);
 	    }
-	    params = FALSE;
 	    percent = FALSE;
 	    break;
 	case ' ':
@@ -927,14 +926,14 @@ fmt_entry(TERMTYPE2 *tterm,
 
     len = 12;			/* terminfo file-header */
 
-    if (pred == 0) {
+    if (pred == NULL) {
 	cur_type = tterm;
 	pred = dump_predicate;
     }
 
-    strcpy_DYN(&outbuf, 0);
+    strcpy_DYN(&outbuf, NULL);
     if (content_only) {
-	column = indent;	/* FIXME: workaround to prevent empty lines */
+	column = indent;	/* workaround to prevent empty lines */
     } else {
 	strcpy_DYN(&outbuf, tterm->term_names);
 
@@ -1119,9 +1118,9 @@ fmt_entry(TERMTYPE2 *tterm,
 			      : ((*srccap == 'k')
 				 ? 0
 				 : has_params(srccap, FALSE)));
-		char *cv = _nc_infotocap(name, srccap, params);
+		const char *cv = _nc_infotocap(name, srccap, params);
 
-		if (cv == 0) {
+		if (cv == NULL) {
 		    if (outform == F_TCONVERR) {
 			_nc_SPRINTF(buffer, _nc_SLIMIT(sizeof(buffer))
 				    "%s=!!! %s WILL NOT CONVERT !!!",
@@ -1130,7 +1129,8 @@ fmt_entry(TERMTYPE2 *tterm,
 		    } else if (suppress_untranslatable) {
 			continue;
 		    } else {
-			char *s = srccap, *d = buffer;
+			const char *s = srccap;
+			char *d = buffer;
 			int need = 3 + (int) strlen(name);
 			while ((*d = *s++) != 0) {
 			    if ((d - buffer + 2) >= (int) sizeof(buffer)) {
@@ -1169,7 +1169,7 @@ fmt_entry(TERMTYPE2 *tterm,
 		char *src = _nc_tic_expand(capability,
 					   outform == F_TERMINFO, numbers);
 
-		strcpy_DYN(&tmpbuf, 0);
+		strcpy_DYN(&tmpbuf, NULL);
 		strcpy_DYN(&tmpbuf, name);
 		strcpy_DYN(&tmpbuf, "=");
 		if (pretty
@@ -1209,9 +1209,10 @@ fmt_entry(TERMTYPE2 *tterm,
     } else if (tversion == V_AIX) {
 	if (VALID_STRING(acs_chars)) {
 	    bool box_ok = TRUE;
-	    const char *acstrans = "lqkxjmwuvtn";
+	    static const char acstrans[] = "lqkxjmwuvtn";
 	    const char *cp;
-	    char *tp, *sp, boxchars[11];
+	    const char *sp;
+	    char *tp, boxchars[sizeof(acstrans)];
 
 	    tp = boxchars;
 	    for (cp = acstrans; *cp; cp++) {
@@ -1223,7 +1224,7 @@ fmt_entry(TERMTYPE2 *tterm,
 		    break;
 		}
 	    }
-	    tp[0] = '\0';
+	    *tp = '\0';
 
 	    if (box_ok) {
 		char *tmp = _nc_tic_expand(boxchars,
@@ -1312,7 +1313,7 @@ fmt_entry(TERMTYPE2 *tterm,
 }
 
 static bool
-kill_string(TERMTYPE2 *tterm, char *cap)
+kill_string(TERMTYPE2 *tterm, const char *const cap)
 {
     unsigned n;
     for (n = 0; n < NUM_STRINGS(tterm); ++n) {
@@ -1325,7 +1326,7 @@ kill_string(TERMTYPE2 *tterm, char *cap)
 }
 
 static char *
-find_string(TERMTYPE2 *tterm, char *name)
+find_string(TERMTYPE2 *tterm, const char *name)
 {
     PredIdx n;
     for (n = 0; n < NUM_STRINGS(tterm); ++n) {
@@ -1353,7 +1354,7 @@ kill_labels(TERMTYPE2 *tterm, int target)
     char name[20];
 
     for (n = 0; n <= 10; ++n) {
-	char *cap;
+	const char *cap;
 
 	_nc_SPRINTF(name, _nc_SLIMIT(sizeof(name)) "lf%d", n);
 	cap = find_string(tterm, name);
@@ -1380,7 +1381,7 @@ kill_fkeys(TERMTYPE2 *tterm, int target)
     char name[20];
 
     for (n = 60; n >= 0; --n) {
-	char *cap;
+	const char *cap;
 
 	_nc_SPRINTF(name, _nc_SLIMIT(sizeof(name)) "kf%d", n);
 	cap = find_string(tterm, name);
@@ -1399,7 +1400,7 @@ kill_fkeys(TERMTYPE2 *tterm, int target)
  * Check if the given acsc string is a 1-1 mapping, i.e., just-like-vt100.
  * Also, since this is for termcap, we only care about the line-drawing map.
  */
-#define isLine(c) (strchr("lmkjtuvwqxn", c) != 0)
+#define isLine(c) (strchr("lmkjtuvwqxn", c) != NULL)
 
 static bool
 one_one_mapping(const char *mapping)
@@ -1445,7 +1446,7 @@ purged_acs(TERMTYPE2 *tterm)
 }
 
 static void
-encode_b64(char *target, char *source, unsigned state, int *saved)
+encode_b64(char *target, const char *source, unsigned state, int *saved)
 {
     /* RFC-4648 */
     static const char data[] =
@@ -1510,7 +1511,7 @@ dump_entry(TERMTYPE2 *tterm,
 		}
 	    }
 	    if (quickdump & 2) {
-		static char padding[] =
+		static const char padding[] =
 		{0, 0};
 		int value = 0;
 
@@ -1554,6 +1555,7 @@ dump_entry(TERMTYPE2 *tterm,
     save_sgr = set_attributes;
 
     if ((FMT_ENTRY() > critlen)
+	&& TcOutput()
 	&& limited) {
 
 	save_tterm = *tterm;
@@ -1652,15 +1654,25 @@ dump_entry(TERMTYPE2 *tterm,
 }
 
 void
-dump_uses(const char *name, bool infodump)
+dump_uses(const char *value, bool infodump)
 /* dump "use=" clauses in the appropriate format */
 {
-    char buffer[MAX_TERMINFO_LENGTH];
+    char buffer[MAX_TERMINFO_LENGTH + EXTRA_CAP];
+    int limit = (VALID_STRING(value) ? (int) strlen(value) : 0);
+    const char *cap = infodump ? "use" : "tc";
 
     if (TcOutput())
 	trim_trailing();
+    if (limit == 0) {
+	_nc_warning("empty \"%s\" field", cap);
+	value = "";
+    } else if (limit > MAX_ALIAS) {
+	_nc_warning("\"%s\" field too long (%d), limit to %d",
+		    cap, limit, MAX_ALIAS);
+	limit = MAX_ALIAS;
+    }
     _nc_SPRINTF(buffer, _nc_SLIMIT(sizeof(buffer))
-		"%s%s", infodump ? "use=" : "tc=", name);
+		"%s=%.*s", cap, limit, value);
     wrap_concat1(buffer);
 }
 
@@ -1683,7 +1695,7 @@ show_entry(void)
 		outbuf.used = (size_t) j;
 	    } else if (!infodump && ch == '\\') {
 		outbuf.used = (size_t) j;
-	    } else if (ch == delim && (j == 0 || outbuf.text[j - 1] != '\\')) {
+	    } else if (ch == delim && (outbuf.text[j - 1] != '\\')) {
 		outbuf.used = (size_t) (j + 1);
 	    } else {
 		break;
@@ -1691,7 +1703,7 @@ show_entry(void)
 	}
 	outbuf.text[outbuf.used] = '\0';
     }
-    if (outbuf.text != 0) {
+    if (outbuf.text != NULL) {
 	(void) fputs(outbuf.text, stdout);
 	putchar('\n');
     }
