@@ -30,7 +30,7 @@
 /****************************************************************************
  *  Author: Thomas E. Dickey                    1996-on                     *
  ****************************************************************************/
-/* $Id: test.priv.h,v 1.224 2025/07/05 15:16:13 tom Exp $ */
+/* $Id: test.priv.h,v 1.225 2025/08/09 01:04:25 tom Exp $ */
 
 #ifndef __TEST_PRIV_H
 #define __TEST_PRIV_H 1
@@ -1088,7 +1088,7 @@ extern int TABSIZE;
 #define SIGKILL 9
 #define getlogin() "username"
 
-#elif defined(EXP_WIN32_DRIVER)
+#else
 
 #if defined(HAVE_NCURSESW_NCURSES_H)
 #include <ncursesw/nc_win32.h>
@@ -1098,16 +1098,6 @@ extern int TABSIZE;
 #include <nc_win32.h>
 #endif
 
-#else
-
-#if defined(HAVE_NCURSESW_NCURSES_H)
-#include <ncursesw/nc_mingw.h>
-#elif defined(HAVE_NCURSES_NCURSES_H)
-#include <ncurses/nc_mingw.h>
-#else
-#include <nc_mingw.h>
-#endif
-
 #endif
 
 /* conflicts in test/firstlast.c */
